from tsunami import web
from tsunami.utils import get_appname
from functools import wraps


def route(path, pattern=None):

    def decorator(cls):
        _path = path
        _appname = get_appname(cls)
        version = getattr(cls, '__VERSION__', 'v1.0')
        if pattern:
            _path = pattern.format(
                path=path,
                appname=_appname,
                version=version)
        _path = _path.replace('//', '/')
        cls.__routes__ = True
        cls.__url__ = _path
        cls.__appname__ = _appname
        return cls

    return decorator



import os
import sys


class Settings(object):

    def __init__(self):
        sys.path.insert(0, os.getcwd())
        settings = __import__('settings')
        for _ in dir(settings):
            if not (_.startswith('__') and _.endswith('__')):
                setattr(self, _, getattr(settings, _))

try:
    settings = Settings()
except ModuleNotFoundError:
    settings = None


__ALL__ = [
    settings
]



from tsunami.core.management.base import BaseCommand
from tsunami.utils import list_appnames
from tsunami.conf import settings
import os
import sys
import pkgutil
import unittest


class Command(BaseCommand):

    def _get_tests(self, *appnames):
        if 'all' in appnames:
            return unittest.defaultTestLoader.discover(
                    'apps/', top_level_dir=settings.ROOT_DIR)
        else:
            suite = unittest.suite.TestSuite()
            for app in appnames:
                suite.addTests(
                    unittest.defaultTestLoader.discover(
                        f'apps/{app}',
                        top_level_dir=settings.ROOT_DIR))
            return suite

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            'modules', metavar='name',
            nargs='+',
            help='Name of app',
            default=['all']
        )

    def execute(self, **options):
        unittest.TextTestRunner().run(
            self._get_tests(*options.get('modules')))



from tsunami.core import response as _resp
from tsunami.core.exceptions import APIException
from tsunami import web
from functools import wraps
import traceback


def standard_api(func):

    @wraps(func)
    async def decorator(*args, **kwargs):
        try:
            result = await func(*args, **kwargs)
            if isinstance(result, dict) or result is None:
                response = {'code': _resp.CODE_0_SUCCESS}
                response.update(result or {})
                return web.json_response(response)
            else:
                return result
        except (APIException, Exception) as e:
            # print(traceback.format_exc())
            if not hasattr(e, 'code'):
                print(traceback.format_exc())
                e.code = 1
            return web.json_response({'code': e.code, 'error_msg': str(e)})

    return decorator



from uvtor.conf import settings
from email import encoders
from email.header import Header
from email.mime.audio import MIMEAudio
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

import mimetypes
import re
import smtplib
import aiosmtplib
import logging


async def send_email(mail_subject, mail_content, receiver_list, cc_list=[]):
    sender = settings.EMAIL_USERNAME
    username = settings.EMAIL_USERNAME
    password = settings.EMAIL_PASSWORD
    smtp = aiosmtplib.SMTP(hostname=settings.EMAIL_SMTP_SERVER)
    await smtp.connect()
    await smtp.login(username, password)
    msg = MIMEText(mail_content)
    msg['Subject'] = Header(mail_subject, 'utf-8')
    msg['From'] = username
    msg['To'] = ','.join(receiver_list)
    msg['CC'] = ','.join(cc_list)
    await smtp.send_message(msg)
    logging.debug(f'Email `{mail_subject}` is sent successfully')
    await smtp.quit()



from tsunami.conf import settings
from tsunami.core.handlers import BaseHandler
from tsunami.utils import list_appnames
from tsunami import web
import pkgutil
import importlib
import inspect


__METHODS = [
    'get', 'post', 'delete', 'put', 'head', 'connect', 'options', 'trace']


def make_app(appname, autoreload=True):
    app = web.Application()
    [
        auto_discover(
            appname, app
        ) for appname in (
            list_appnames() if appname == 'all' else [appname])
    ]
    return app


def get_routes_by_cls(cls):
    if getattr(cls, '__routes__', False):
        _c = cls()
        routes_ = []
        print(f'Add routes ({cls.__url__}, {cls})')
        for _m in __METHODS:
            routes_.append(
                web.route(_m, _c.__url__, getattr(_c, _m)))
        return routes_
    return []


def auto_discover(appname, app):
    handlers = importlib.import_module('apps.{}.handlers'.format(appname))

    for loader, modname, ispkg in pkgutil.walk_packages(handlers.__path__):
        _module = loader.find_module(modname).load_module(modname)
        [
            app.add_routes(get_routes_by_cls(cls))
                for name, cls in inspect.getmembers(_module)
                    if inspect.isclass(cls) and issubclass(cls, BaseHandler)
        ]



from tsunami import web
from tsunami.core.management.base import BaseCommand, CommandError
from tsunami.core import make_app
from aiohttp import GunicornUVLoopWebWorker
from gunicorn.six import iteritems
import asyncio
import uvloop
import os
import logging
import gunicorn.app.base


asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())


class TsunamiGunicornApplication(gunicorn.app.base.BaseApplication):

    def __init__(self, app, options=None):
        self.options = options or {}
        self.application = app
        super(TsunamiGunicornApplication, self).__init__()

    def load_config(self):
        config = dict([(key, value) for key, value in iteritems(self.options)
                       if key in self.cfg.settings and value is not None])
        for key, value in iteritems(config):
            self.cfg.set(key.lower(), value)

    def load(self):
        return self.application


class Command(BaseCommand):

    def check(self):
        if not os.path.exists('.tsunami'):
            raise CommandError('Invalid tsunami project')

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            'name', metavar='name',
            help='Name of app',
        )
        parser.add_argument(
            '--port', '-P', metavar='port',
            default=8000,
            help='Listen port',
        )

        parser.add_argument(
            '--logging', '-L', metavar='port',
            default='INFO',
            help='Logging Level',
        )

        parser.add_argument(
            '--address', '-H', metavar='address',
            default='127.0.0.1',
            help='Listen address',
        )

        parser.add_argument(
            '--mode', '-M', metavar='mode',
            default='default',
            choices=['default', 'gunicorn'],
            help='Run directly or by gunicorn',
        )

        parser.add_argument(
            '--gunicorn-args', '-G', metavar='argA:valA;argB:valB',
            help=(
                'gunicorn arguments, '
                'only available on mode `gunicorn`')
        )

    def execute(self, **options):
        _appname = options.get('name')
        _address = options.get('address')
        _port = options.get('port')
        _logging = options.get('logging')
        _mode = options.get('mode')
        _gargs = options.get('gunicorn_args')

        logging.info(f'Server started, listen on {_address}:{_port}')
        app = make_app(_appname)
        if _mode == 'default':
            web.run_app(app, port=_port)
        elif _mode == 'gunicorn':
            _gconf = {
                'worker_class': 'aiohttp.GunicornUVLoopWebWorker',
                'bind': f'{_address}:{_port}'
            }
            if _gargs:
                _gconf.update(
                    dict([kv.split(':') for kv in _gargs.split(';')]))

            TsunamiGunicornApplication(
                app,
                _gconf
            ).run()



from tsunami.core.management.base import BaseCommand, CommandError
import os
import shutil
import re
import tsunami


class Command(BaseCommand):

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            'name', metavar='name',
            help='Name of app',
        )

    def check(self):
        if not os.path.exists('.tsunami'):
            raise CommandError('Invalid tsunami project')

    def _is_valid_name(self, name):
        if not re.match(r'^[a-zA-Z]+[\w_]+$', name):
            raise CommandError('Invalid project name')

    def _init_handler(self, _appname):
        for dir_name, subdir_list, file_list in os.walk(
                os.path.join('apps', _appname)):
            for _file in file_list:
                if not _file.endswith('.tpl'):
                    continue
                _path = os.path.join(dir_name, _file[:-4])
                shutil.move(
                    os.path.join(dir_name, _file),
                    _path,
                    )
                if _file == 'main.py.tpl':
                    with open(_path) as _f:
                        _handler = _f.read()
                    _handler = _handler.format(
                        appname=_appname
                        )
                    with open(_path, 'w') as _f:
                        _f.write(_handler)

    def execute(self, **options):
        _appname = options.get('name')
        self._is_valid_name(_appname)
        _template_dir = os.path.join(
            tsunami.__path__[0], '_templates', 'app_template')

        shutil.copytree(_template_dir, os.path.join('apps', _appname))

        self._init_handler(_appname)

        print(f'Successfully create app {_appname}')



from tsunami.conf import settings
import inspect
import os


def get_appname(cls):
    _cpath = inspect.getfile(cls)
    _common_prefix = os.path.commonprefix(
        [
            _cpath,
            settings.ROOT_DIR
        ])
    return os.path.relpath(
        _cpath, _common_prefix)[5: ].split('/')[0]


def list_appnames():
    return [
        f for f in os.listdir(
            os.path.join(settings.ROOT_DIR, 'apps')
    ) if not f.startswith('_')]



from tsunami import test_utils
from tsunami import web
from tsunami.utils import get_appname
from tsunami.core import make_app


class BaseTestCase(test_utils.AioHTTPTestCase):

    async def get_application(self):

        appname = get_appname(self.__class__)
        app = make_app(appname)
        return app



from tsunami.core import response as _resp


class APIException(Exception):

    code = _resp.CODE_1_ERROR
    default_detail = 'A server error occurred.'

    def __init__(self, detail=None):
        if detail is not None:
            self.detail = detail
        else:
            self.detail = self.default_detail

    def __str__(self):
        return self.detail



from tsunami.conf import settings
from tsunami.core.cache import cache
from tsunami import web
import functools
import pickle


class BaseHandler:

    async def get(self, request):
        raise web.HTTPMethodNotAllowed('get', '*')

    async def post(self, request):
        raise web.HTTPMethodNotAllowed('post', '*')

    async def delete(self, request):
        raise web.HTTPMethodNotAllowed('delete', '*')

    async def put(self, request):
        raise web.HTTPMethodNotAllowed('put', '*')

    async def head(self, request):
        raise web.HTTPMethodNotAllowed('head', '*')

    async def options(self, request):
        raise web.HTTPMethodNotAllowed('options', '*')

    async def trace(self, request):
        raise web.HTTPMethodNotAllowed('trace', '*')

    async def connect(self, request):
        raise web.HTTPMethodNotAllowed('connect', '*')




